( 3iOLED.fs -- 2020-10-08---Thu---21:02 )
( This software module runs an OLDE display from atop 3iOLED.avr which uses 3iForth.avr )
( Hardware: Arduino Nano using I2C 2-pin serial comms )


\  Hardware: SSD1306 display chip for OLED 32x- 64x- or 128x32 display, connected to I2C address 0x3C
\  Connectivity: 2-pins for I2C comms on Arduino Nano: SCL=A5 (19), SDA=A4 (18), plus power VCC +5VDC and Gnd

\ exported function constants
$F  constant CALL_DISPLAYCLEAR
$10 constant CALL_TEXTSIZE
$11 constant CALL_TEXTCOLOR
$12 constant CALL_CURSOR
$13 constant CALL_PRINT
$14 constant CALL_DISPLAYSHOW
$15 constant CALL_PIXEL
$16 constant CALL_CHAR



\ ==== ADAFRUIT SSD1306 DRIVER SOFTWARE PRIMITIVES for ARDUINO ====
: oled.  ( -- )            CALL_DISPLAYSHOW xcall ;    \ required after every command to update display
: [oled-clear]  ( -- )     CALL_DISPLAYCLEAR xcall ;
: oled-clear ( -- ) [oled-clear] oled. ;

0 constant BLACK    
1 constant WHITE    
2 constant INVERSE
variable OCOLOR 
: ocolor! ( n -- ) ocolor c! ;
: ocolor@ ( -- n ) ocolor c@ ;
: [oled-color]  ( color -- ) dup ocolor! xstore CALL_TEXTCOLOR xcall ;
white ocolor!  \ default
\ screen dimensions in pixels
$7F constant XMAX  \ 127
$3F constant YMAX  \ 63 or 31
: [oled-cursor] ( y x )    swap xstore xstore CALL_CURSOR xcall ;  \ position cursor
: [oled-pixel]  ( y x color ) -rot swap xstore xstore xstore CALL_PIXEL xcall ;
: [oled-dot]  ( y x -- ) ocolor@ [oled-pixel] ;  
: oled-dot  ( y x -- ) [oled-dot] oled. ;  \ thinnest pixel
: oled-dot2x2 ( y x -- ) 2dup [oled-dot] 2dup 1 + [oled-dot] swap 1 + swap 2dup [oled-dot] 1 + oled-dot ; \ a thicker pixel
: oled-flashdot ( y x -- ) inverse ocolor! 2dup oled-dot 20 ms oled-dot ;
: oled-flashdot2x2 ( y x -- ) inverse ocolor! 2dup oled-dot2x2 20 ms oled-dot2x2 ;
: pixel-test 2dup oled-dot 1500 ms oled-dot2x2  1000 ms ;
\ TODO: convert to machine primitives to avoid latency on serial line
: [hline] ( y -- ) XMAX 1 + 0 do dup i [oled-dot] loop drop ; \ oled.
: [vline] ( x -- ) YMAX 1 + 0 do i over [oled-dot] loop drop ;
: hline ( y -- ) XMAX 1 + 0 do dup i [oled-dot] loop drop oled. ;
: vline ( x -- ) YMAX 1 + 0 do i over [oled-dot] loop drop oled. ;

1 constant SMALLEST
5 constant LARGEST
: oled-textsz ( sz -- )  xstore CALL_TEXTSIZE xcall ;  \ 1-5, 5 largest, 1 smallest
: [oled-reset] ( -- ) ocolor@ [oled-color] 1 oled-textsz 0 0 [oled-cursor] ;
: oled-init  [oled-reset] oled-clear ;
: erase-test  oled-init 500 ms  white ocolor! 30 15 pixel-test black ocolor! 30 15 pixel-test ;

: [oled-char]   ( ch -- ) xstore CALL_CHAR xcall ;
: oled-char   ( ch -- ) [oled-char] oled. ;
: hello  'h' [oled-char] 'e' [oled-char] 'l' [oled-char] 'l' [oled-char] 'o' [oled-char] oled. ;
: [oled-string] ( c-adr n -- ) 0 do c@+ [oled-char] loop drop ;
: oled-string ( cadr n -- ) [oled-string] oled. ;

\ : oled-builtin  ( -- )     CALL_PRINT xcall ; \ runs built-in test function

\ ==== APPLICATIONS ====
: note. 14 oled-char ;
: rest. 22 oled-char ;
: space. 32 oled-char ;
: minus. 45 oled-char ; 
: rarrow. 16 oled-char ;
: oarrow. 111 oled-char ;
: larrow. 17 oled-char ;
: plus. 43 oled-char ; 
: cr.   10 oled-char ;
: cross. 88 oled-char ;
: star. 42 oled-char ;


\ SOFTWARE MENUS: 
\ - < o > +
: menu5  oled-init 2 oled-textsz minus. space. larrow. space. oarrow. space. rarrow. space. plus. cr.  ;
\ ESC < OK > +
\ BACK < OK > +
\ X < TICK > +

\ SOFTWARE DICE:
\ need random number roller
\ APPLICATION: D&D / Fighting Fantasy accompaniment...
: dice6.  oled-init s" * * *" oled-string cr. cr. s" * * *" oled-string ;
: dice5.  oled-init s" *   *" oled-string cr. s"   *  " oled-string cr. s" *   *" oled-string ;
: dice4.  oled-init s"  * * " oled-string cr. cr. s"  * * " oled-string ;
: dice3.  oled-init s" *" oled-string cr. s"   *" oled-string cr. s"     *" oled-string ;
: dice2.  oled-init cr. s"  * * " oled-string ;
: dice1.  oled-init cr. s"  *" oled-string ;


\ SOFTWARE BANNER:
: score ( -- ) 5   5 0 do 10 + dup [hline] loop drop oled. ;  \ APPLICATION: musical score!
: quarters ( -- ) YMAX 2 / [hline]    XMAX 2 / [vline] ;

: love  s" I love you very much!" oled-string ;
: big-love oled-init 5 oled-textsz s" love" oled-string ;
: big-adam oled-init 5 oled-textsz s" Adam" oled-string ;
: big-word ( cadr n -- ) 5 oled-textsz oled-init oled-string ;  \  s" Hi!" big-word
: med-word ( cadr n -- ) 3 oled-textsz oled-init oled-string ;  \  s" Hello!" med-word
: small-word ( cadr n -- ) 1 oled-textsz oled-init oled-string ;

\ IDEA: Text-i-talkie --- allow sending text messages using wireless version of Arduino Nano :)
: sz ( sz -- ) oled-init oled-textsz ;
: xword ( cadr n -- ) oled-string  ;
: xcr ( -- ) $A oled-char ;
: xclear ( -- ) oled-init ;

\ EXERCISE: do the maths for all kinds of shapes (lines, circles, ellipses)
\ Fixed point arithmetic

\ *** Etch-a-Sketch APPLICATION in 7 LINES of forth code :) ***
\ etch-a-sketch drawing controls, and interpolation between points...
\ connect D4 to LED with 220R resistor, then on to pin A7 (21) through 1.5KR resistor and to ground through 10K potentiometer
\ tactile button (pullup) at pin 5

: etch-init  5 pullup  14  pullup 20 ms ;  \ reset screen pin,  pen-up / down pin 
\ 10 bit ADC reference
: Xref-calc  1024 xmax / 1 - ;  \ works with 1K resistors, higher value resistors then limit the range of drawing motion from pots
Xref-calc constant Xref
: Yref-calc  1024 ymax / 1 - ;
Yref-calc constant Yref
: readXlevel  20 pinreadLevel Xref / ; 
: readYlevel  21 pinreadLevel Yref / ; 
: xclear? ( --) 5 pinread 0= if oled-init 20 ms then ;
: erase?  ( -- ) 14 pinread 0= if black ocolor! else white ocolor! then ;
: penup? ( -- ) 14 pinread 0= if oled-flashdot else oled-dot then ;
: etch oled-init begin  erase? readYlevel readXlevel penup? xclear? key? until ;

\ oscilloscope - scrolling in Y direction and plotting the level, refreshing the page when screen filled (later do scrolling)
: scroll? ( y -- y ) dup 0= if oled-init 20 ms then ;
: oled-level 0 begin scroll? dup readXlevel oled-dot 1 + ymax mod key?  until drop ;

\ EXERCISE: joystick is just 2 10k pots 
\ EXERCISE: add functionality within .AVR to read more efficiently for better responsiveness.
\ EXERCISE: reduce the nonlinearity of the pot action...


\ ============================================================

: help-oled
  cr
  cr  ." OLED DISPLAY (help-oled)"
  cr  
  cr ." uC hardware: Arduino Nano pins 18/A4=SDA, 19/A5=SCL to OLED I2C display"
  cr ." display: 128x64 pixel OLED i2c display with SSD1306 driver IC (max 10-15mA @5V)"
  cr ." firmware: 3iOLED.avr on 3iForth.avr"
  cr ." software: include 3iForth.fs, 3iOLED.fs"
  cr 
  cr 
;

help-oled


\ Version History - Started: 2020-10-08---Thu---21:02
\ v1 - 2020-10-09---Fri
\        FEATURE: Completed OLED display enablement, positioning, and pixel drawing
\ v2 - 2020-10-10---Sat---15:21
\       FEATURE: Completed character and string writing
\       FEATURE: Completed 2-dimensional OLED etch-a-sketch with two potentiometers!
\       FEATURE: Completed scrolling oscilloscope with 1-dimensional OLED reading pot
\ v3 - 2021-05-27---Thu---08:02 -- overhaul of primitives - tidying up the structure of the vocabulary
\ V4 - 2021-06-03---Thu---03:36 -- prototyped pen-up and erase features for oled-etch application (20 min each in Forth)
